(* AxCmps - ActiveX non-visual component development support in Delphi
 * Copyright (c) 2003 by Mandys Tomas-MandySoft
 *)

 { URL: http://www.2p.cz }

unit AxCmps;

interface
uses
  SysUtils, Classes, Controls, ActiveX, AxCtrls, AxCtrlsReg, Windows, ComServ, ComObj, Graphics;

type
  TActiveXComponentControl = class;

  TActiveXComponent = class(TActiveXControl, IOleControl)
  private
    fUserMode: WordBool;
  protected
    procedure ModeChanged; virtual;
    procedure InitializeControl; override;
    procedure ActiveXLoaded; virtual;
    { IOleControl }
    function GetControlInfo(var ci: TControlInfo): HResult; stdcall;
    function OnMnemonic(msg: PMsg): HResult; stdcall;
    function OnAmbientPropertyChange(dispid: TDispID): HResult; stdcall;
    function FreezeEvents(bFreeze: BOOL): HResult; stdcall;
  public
    property UserMode: WordBool read fUserMode;
    procedure Initialize; override;
  end;

  TActiveXComponentControl = class(TCustomControl)
  private
    fBitmapId: Integer;
    procedure SetRunTime(const Value: Boolean);
    function GetRunTime: Boolean;
  protected
    procedure Paint; override;
    procedure DesignPaint; virtual;
  public
    property BitmapId: Integer read fBitmapId write fBitmapId;
    constructor Create(AOwner: TComponent); override;
    property RunTime: Boolean read GetRunTime write SetRunTime;
  end;

  TActiveXComponentFactory = class(TRegActiveXControlFactory)
  public
    constructor Create(ComServer: TComServerObject;
      ActiveXControlClass: TActiveXControlClass;
      const ClassID: TGUID;
      ToolboxBitmapID: Integer; const LicStr: string; MiscStatus: Integer;
      ThreadingModel: TThreadingModel = tmSingle);
  end;

implementation

{ TActiveXComponent }

procedure TActiveXComponent.Initialize;
begin
  fUserMode:= True;  // VB does not call OnAmbientPropertyChange, so it must be runtime default option
  inherited;
  ModeChanged;
end;

procedure TActiveXComponent.InitializeControl;
begin
  inherited;
{  if (Control <> nil) and (Control is TActiveXComponentControl) then
    TActiveXComponentControl(Control).BitmapId:= ; // default is 1 //UpperCase(Self.ClassName); }
end;

procedure TActiveXComponent.ModeChanged;
begin
  if (Control <> nil) and (Control is TActiveXComponentControl) then
    begin
      TActiveXComponentControl(Control).RunTime := fUserMode;
    end;
end;

function TActiveXComponent.GetControlInfo(var ci: TControlInfo): HResult;
begin
  Result := inherited GetControlInfo(ci);
end;

function TActiveXComponent.OnMnemonic(msg: PMsg): HResult;
begin
  Result := inherited OnMnemonic(msg);
end;

function TActiveXComponent.OnAmbientPropertyChange(
  dispid: TDispID): HResult;
var
  AD: IAmbientDispatch;
begin
  Result := inherited OnAmbientPropertyChange(dispid);
  if (dispid = DISPID_UNKNOWN) or (dispid = DISPID_AMBIENT_USERMODE) then  // called only by Delphi
    if (ClientSite <> nil) and (ClientSite.QueryInterface(IAmbientDispatch, AD) = S_OK) then
    begin
      fUserMode:= AD.UserMode;
      ModeChanged;
    end;
end;

function TActiveXComponent.FreezeEvents(bFreeze: BOOL): HResult;
begin
  Result := inherited FreezeEvents(bFreeze);
  if not bFreeze then
    ActiveXLoaded;   // for VB - according http://www.geocities.com/nafmis/FAQ/COMFAQ199907.html
end;

procedure TActiveXComponent.ActiveXLoaded;
begin
end;

{ TActiveXComponentControl }

constructor TActiveXComponentControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if AOwner is TWinControl then
    Parent := (AOwner as TWinControl);
  Width := 26;
  Height := 26;
  ControlStyle:= [csOpaque, csFixedWidth, csFixedHeight, csNoStdEvents {, csReplicatable, {csNoDesignVisible}];
  Constraints.MaxHeight:= Width;
  Constraints.MinHeight:= Width;
  Constraints.MaxHeight:= Height;
  Constraints.MinHeight:= Height;
  Visible:= False;
  fBitmapId:= 1;
end;

procedure TActiveXComponentControl.Paint;
begin
  inherited Paint;
//  if not RunTime then   // visibility is controled by ActiveX,
//  if tested here - MS Visual *, Excel does not show icon in designed if project being reloaded (even placing works OK)
    DesignPaint;
end;

procedure TActiveXComponentControl.DesignPaint;
var
  PaintRect: TRect;
  Bmp: TBitmap;
begin
  PaintRect := Rect(1, 1, Width, Height);
  Bmp:= TBitmap.Create;
  try
    Bmp.LoadFromResourceId(hInstance, fBitmapId);
    Canvas.StretchDraw(PaintRect, Bmp);
  finally
    Bmp.Free;
  end;
{  Canvas.Pen.Style := psDot;
  Canvas.Brush.Style := bsClear;
  Canvas.Rectangle( 0, 0, Width, Height); }
end;

procedure TActiveXComponentControl.SetRunTime(const Value: Boolean);
begin
  Visible:= not Value;
  SetDesigning(not Value, True);
  Repaint;
end;

function TActiveXComponentControl.GetRunTime: Boolean;
begin
  Result:= not (csDesigning in ComponentState);
end;

{ TActiveXComponentFactory }

constructor TActiveXComponentFactory.Create(ComServer: TComServerObject;
  ActiveXControlClass: TActiveXControlClass; const ClassID: TGUID;
  ToolboxBitmapID: Integer; const LicStr: string; MiscStatus: Integer;
  ThreadingModel: TThreadingModel);
begin
  inherited Create(ComServer, ActiveXControlClass, TActiveXComponentControl, ClassID, ToolboxBitmapID, LicStr,
  MiscStatus or OLEMISC_SIMPLEFRAME or OLEMISC_ACTSLIKELABEL or OLEMISC_INVISIBLEATRUNTIME,
  ThreadingModel);
end;

end.
