unit DemoEurotelM2MXFr;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, EurotelM2MX_TLB,
  StdCtrls, Terminal, StdVcl, AxCtrls, ExtCtrls, OleCtrls, AxCmps, OleCmps;

type
  TForm1 = class(TForm)
    Log1: TTerminal;
    Button1: TButton;
    Button2: TButton;
    Memo1: TMemo;
    Label1: TLabel;
    Button3: TButton;
    Label2: TLabel;
    Label3: TLabel;
    Phone: TEdit;
    Msg: TEdit;
    Button4: TButton;
    CheckBox1: TCheckBox;
    Shape1: TShape;
    BeforeAck: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
  private
    fM2MClient: TEurotelM2MClientX;
    procedure OnBusyChanged(Sender: TObject; aBusy: WordBool);
    procedure OnLog(Sender: TObject; Channel: Byte; const Buff: WideString);
    procedure OnProcessM2MRequest(ASender: TObject;
      var aPlugin: IEurotelM2MPlugin);
    procedure OnReceivedM2MRequest(ASender: TObject;
      var aPlugin: IEurotelM2MPlugin; var aProcessed: TEurotelM2MProcessedFlag);
    procedure LogItem(aItem: IEurotelM2MPlugin);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation
uses
  CmdLine, AuxProj{, Variants};

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var
  S: string;
begin
  UpdateIni:= False;
  fM2MClient:= TEurotelM2MClientX.Create(Self);
  fM2MClient.Left:= 100;
  fM2MClient.OnBusyChanged:= OnBusyChanged;
  fM2MClient.OnLog:= OnLog;
  fM2MClient.OnReceivedM2MRequest:= OnReceivedM2MRequest;
  fM2MClient.OnProcessM2MRequest:= OnProcessM2MRequest;
  S:= '';
  GetCmdString('', clUpcase, S);
  if S <> '' then
  begin
    Ini.ReadSectionValues(S, Memo1.Lines);
    Memo1.Lines.Insert(0, Format('ReceiverTimeout=%d', [Ini.ReadInteger(S+'-Client', 'ReceiverTimeout', 0)]));
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  fM2MClient:= nil;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  fM2MClient.ReceiverTimeout:= StrToIntDef(Memo1.Lines.Values['ReceiverTimeout'], 0)*1000;
  fM2MClient.Params:= TStringsAdapter.Create(Memo1.Lines) as IStrings;
  fM2MClient.Active:= True;
  if not fM2MClient.Active then
    Log1.LogErr('Error', fM2MClient.LastErrorStr)
  else
    Log1.Log('Opened');
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  fM2MClient.Active:= False;
  if not fM2MClient.Active then
    Log1.Log('Closed');
end;

procedure TForm1.OnLog(Sender: TObject; Channel: Byte;
  const Buff: WideString);
begin
  if CheckBox1.Checked then
    Log1.Log(Buff);
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  fM2MClient.ForcePolling(False);
//  Log1.Log(Format('Connection: %d', [Byte(fEMIClient.TestConnection)]));
end;

procedure TForm1.OnBusyChanged(Sender: TObject; aBusy: WordBool);
const
  C: array [Boolean] of TColor = (clLime, clRed);
begin
  Shape1.Brush.Color:= C[aBusy];
  Application.ProcessMessages;  // only to update VCL
end;

procedure TForm1.Button4Click(Sender: TObject);
var
  It: IEurotelM2MSMSSubmit;
begin
  It:= CoEurotelM2MSMSSubmit.Create;
  try
    It.Data:= Msg.Text;
    It.DA:= Phone.Text;
    It.ReportRequest:= em2msrrAll;
    if not fM2MClient.SubmitSMS(It) then
      Log1.LogErr('Error', fM2MClient.LastErrorStr)
    else
      Log1.Log(Format('> %s) %s "%s"', [DateTimeToStr(It.Stamp), Phone.Text, Msg.Text]));
  finally
    It:= nil;
  end;
end;

procedure TForm1.LogItem(aItem: IEurotelM2MPlugin);
var
  II: IDispatch;
begin
  try
    if Succeeded(IDispatch(aItem).QueryInterface(IEurotelM2MSMSDeliver, II)) then
      begin
        Log1.Log(Format('< %s) %s "%s"', [DateTimeToStr(aItem.Stamp), IEurotelM2MSMSDeliver(aItem).OA, IEurotelM2MSMSDeliver(aItem).Data]));
      end
    else if Succeeded(IDispatch(aItem).QueryInterface(IEurotelM2MSMSStatusReport, II)) then
      begin
        Log1.Log(Format('! %s) %s Status:%d', [DateTimeToStr(IEurotelM2MSMSStatusReport(aItem).Stamp), IEurotelM2MSMSStatusReport(aItem).OA, Integer(IEurotelM2MSMSStatusReport(aItem).Status)]));
      end
    else if Succeeded(IDispatch(aItem).QueryInterface(IEurotelM2MPlugin, II)) then
      begin
        Log1.Log(Format('? %s) %s', [DateTimeToStr(aItem.Stamp), aItem.Payload]));
      end;
  except
    on E: Exception do
    begin
      try
        Log1.LogErr('Error(Logitem)', E.Message);
      except
      end;
    end;
  end;
end;

procedure TForm1.OnProcessM2MRequest(ASender: TObject;
  var aPlugin: IEurotelM2MPlugin);
begin
  if not BeforeAck.Checked then
    LogItem(aPlugin);
end;

procedure TForm1.OnReceivedM2MRequest(ASender: TObject;
  var aPlugin: IEurotelM2MPlugin; var aProcessed: TEurotelM2MProcessedFlag);
begin
  if BeforeAck.Checked then
  begin
    try
      LogItem(aPlugin);
      aProcessed:= em2mpfProcessed;
    except
      aProcessed:= em2mpfAborted;
    end;
  end;
end;

end.

